****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_cidambltables.sas  
*
* Created (mm/dd/yyyy): 1/3/2018
* Last modified: 
* Version: 1
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*   
*  Program inputs:    
*                                     
*  Program outputs:  
*                                                                                                  
*  PARAMETERS:                                                                       
*            
*  Programming Notes:                                                                                
*                                                                           
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Mini-Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_cidambltables();

%put =====> MACRO CALLED: ms_cidambltables v1.0;

	data _null_;
		call symputx('NUMAGECAT',countw("&AGESTRAT."," "));
		call symputx('NUMENROLLCAT',countw("&enrolldayscat."," "));
		call symputx('YEARFROM',year(&startfollowup.));
		call symputx('YEARTO',year(&enddate.));
	run;
	%put &NUMAGECAT.  &YEARFROM.  &YEARTO.;


	*Mother tables;
	%if %eval(&type.=4) %then %do;

		data _mothers;
			set _PtsMasterList(keep=patid birth_date age encounterID enctype facility_code enc_adate enc_ddate birth_type agegroup 
								rename=patid=mpatid rename=birth_date=mbirth_date rename=enc_adate=adate rename=enc_ddate=ddate);
		run; 

		*summary counts;
		proc sql noprint;
			select count(*), count(distinct mpatid) into :deliveries, :mothers
			from _mothers;
		quit;
		%put &deliveries. &mothers.;

		data msoc.motherdeliveries_infants_counts;
			length level $3;

			level = '001';
			count = &deliveries.;
			output;
			level = '002';
			count = &mothers.;
			output;
		run;


		*square tables;
		data _square1;
			format AgeGroup $9.;
			
			do AgeGroupNum=1 to &NUMAGECAT.;
				do Year=&YEARFROM. to &YEARTO.;		
	 			AgeGroup = scan("&AGESTRAT.",AgeGroupNum,' ');
					EncType="IP";	
					output;
					EncType="IS";	
					output;
					EncType="OA";	
					output;
					EncType="ED";	
					output;
					EncType="AV";	
					output;
				end;
			end;	
		run;

		data _bt;
			length birth_type 3;
			birth_type = 0;
			output;
			birth_type = 1;
			output;
			birth_type = 2;
			output;
			birth_type = 3;
			output;
			birth_type = 4;
			output;
			birth_type = 5;
			output;
			birth_type = 8;
			output;
			birth_type = 9;
			output;
		run;

		proc sql noprint undo_policy=none;
			create table _square1 as
			select x.*, y.*
			from _square1 as x,
			_bt as y;
		quit;

		* Assign AgeGroup number;
		data _age;
	    numgrp = countw("&AGESTRAT."," ") ;
	    do i=1 to numgrp;
	        AgeGroup=scan("&AGESTRAT",i," ");
	        AgeGroupNum=i;
	        output;
	    end;
	    keep AgeGroup AgeGroupNum;
	    run;

		proc sql noprint undo_policy=none;
	    create table _mothers as 
	    select outfile.*,
	           age.AgeGroupNum
	    from _mothers as outfile left join
	         _age as age
	    on age.AgeGroup=outfile.AgeGroup;
	    quit;

		*merge squared table with _infants table;
		data _mothers;
		    set _mothers(in=a)
		        _square1(in=b);
		    if a then do;
		        mother=1;
				year=year(adate);
		    end;
		    else do;
		        mother=0;    
		    end;
		run;

	  	%macro createNumTableEntries(class=,outfile=,level=);

	        *One record per patient for the class stratification;
	        proc means data=_mothers nway noprint missing;
	        var mother;
	        class mPatId &class.;
	        id mother;
	        output out=_num0(drop=_:) sum(mother)=count;             
	        run;

	        *One record per patient for the class stratification;
	        proc means data=_num0 missing nway noprint;
	        var count;
	        class &class.;
	        output out=&outfile.(drop=_:) sum=;             
	        run;
		
		        %IF %length(&class.)=0  %THEN %DO;
		            data _NumCounts;
		            set &outfile.;
		            format level $3. group $30.;
		            Level="&level.";
					group = "&ITGROUP.";
		            run;
		        %END;
		        %ELSE %DO;
		            data _NumCounts;
		            set  _NumCounts
		                &outfile.(in=a);
		                if a then do;	
							Level="&level.";
							group = "&ITGROUP.";
						end;
		            run;
		        %END;
	    %mend;
	    %createNumTableEntries(class=,outfile=_ALL,level=000);

	    *Base Tables (levels will need to be adjusted when integrated to RAF);
	    %createNumTableEntries(class=agegroup agegroupnum,outfile=_AG,Level=001);
	    %createNumTableEntries(class=year,outfile=_Y,Level=002);
	    %createNumTableEntries(class=enctype,outfile=_E,Level=003);
	    %createNumTableEntries(class=agegroup agegroupnum year,outfile=_AG_Y,Level=004);
	    %createNumTableEntries(class=agegroup agegroupnum enctype,outfile=_AG_E,Level=005);
	    %createNumTableEntries(class=year enctype,outfile=_Y_E,Level=006);
	    %createNumTableEntries(class=agegroup agegroupnum year enctype,outfile=_AG_Y_E,Level=007);
	    %createNumTableEntries(class=birth_type,outfile=_bt,Level=008);
	    %createNumTableEntries(class=birth_type agegroup agegroupnum,outfile=_BT_AG,Level=009);
	    %createNumTableEntries(class=birth_type year,outfile=_BT_Y,Level=010);
	    %createNumTableEntries(class=birth_type enctype,outfile=_BT_E,Level=011);
	    %createNumTableEntries(class= birth_type agegroup agegroupnum year enctype,outfile=_BT_AG_Y_E,Level=012);

		proc sort data=_NumCounts;
			by /*group*/ Level agegroupnum year enctype birth_type;
		run; 

		* Output to MSOC;

		    %IF %EVAL(&group.=1)  %THEN %DO;
				data MSOC.&RUNID._deliveries_counts;
					retain /*group*/ Level agegroup year enctype birth_type count;
					set _NumCounts;
					keep /*group*/Level agegroup year enctype birth_type count;
				run;
		    %END;
		    %ELSE %DO;
				proc append base=MSOC.&RUNID._deliveries_counts
		                    data=_NumCounts(keep=/*group*/ Level agegroup year enctype birth_type count) force;
		        run;
		    %END;
	%end;


	*Child tables;
	%if %eval(&type.=1) %then %do;

		data _infants;
			set _PtsMasterList(keep=patid birth_date sex enrolldays:
						rename=patid=cpatid rename=birth_date=cbirth_date);
		run; 
		
		*summary counts;
		proc sql noprint;
			select count(*) into :infants
			from _infants;
		quit;
		%put &infants.;

		data msoc.motherdeliveries_infants_counts;
			set msoc.motherdeliveries_infants_counts end=eof;
			output;
			if eof then do;
				level = '003';
				count = &infants.;
				output;
			end;
		run;

		*square tables;

		data _square1;
			length enrolldays $10;
			
			do enrolldayssort=1 to &NUMENROLLCAT.;
				do Year=&YEARFROM. to &YEARTO.;							
					enrolldays = scan("&enrolldayscat.",enrolldayssort,' ');
						Sex="A";	
						output;
						Sex="F";	
						output;
						Sex="M";
						output;		
						Sex="U";
						output;		
				end;
			end;	
		run;

		*merge squared table with _infants table;
		data _infants;
		    set _infants(in=a)
		        _square1(in=b);
		    if a then do;
		        infant=1;
				year=year(cbirth_date);
		    end;
		    else do;
		        infant=0;    
		    end;
		run;

	  	%macro createNumTableEntries(class=,outfile=,level=);

	        *One record per patient for the class stratification;
	        proc means data=_infants nway noprint missing;
	        var infant;
	        class cPatId &class.;
	        id infant;
	        output out=_num0(drop=_:) sum(infant)=count;             
	        run;

	        *One record per patient for the class stratification;
	        proc means data=_num0 missing nway noprint;
	        var count;
	        class &class.;
	        output out=&outfile.(drop=_:) sum=;             
	        run;
		
		        %IF %length(&class.)=0  %THEN %DO;
		            data _NumCounts;
		            set &outfile.;
		            format level $3. group $30.;
		            Level="&level.";
					group = "&ITGROUP.";
		            run;
		        %END;
		        %ELSE %DO;
		            data _NumCounts;
		            set  _NumCounts
		                &outfile.(in=a);
		                if a then do;	
							Level="&level.";
							group = "&ITGROUP.";
						end;
		            run;
		        %END;
	    %mend;

	    %createNumTableEntries(class=,outfile=_ALL,level=000);

	    *Base Tables (levels will need to be adjusted when integrated to RAF);
	    %createNumTableEntries(class=sex,outfile=_S,Level=001);
	    %createNumTableEntries(class=year,outfile=_Y,Level=002);
	    %createNumTableEntries(class=enrolldays enrolldayssort,outfile=_E,Level=003);
	    %createNumTableEntries(class=sex year,outfile=_S_Y,Level=004);
	    %createNumTableEntries(class=sex enrolldays enrolldayssort,outfile=_S_E,Level=005);
	    %createNumTableEntries(class=year enrolldays enrolldayssort,outfile=_Y_E,Level=006);
	    %createNumTableEntries(class=sex year enrolldays enrolldayssort,outfile=_S_Y_E,Level=007);


		proc sort data=_NumCounts;
			by /*group*/ Level sex year enrolldayssort;
		run; 

		* Output to MSOC;

		    %IF %EVAL(&group.=1)  %THEN %DO;
				data MSOC.&RUNID._infants_counts;
					retain /*group*/ Level sex year enrolldays count;
					set _NumCounts;
					keep /*group*/ Level sex year enrolldays count;
				run;
		    %END;
		    %ELSE %DO;
				proc append base=MSOC.&RUNID._infants_counts
		                    data=_NumCounts(keep=/*group*/ Level sex year enrolldays count) force;
		        run;
		    %END;
	%end;



%put NOTE: ******** END OF MACRO: ms_cidambltables  v1.0 ********;

%mend ms_cidambltables;
